import os
import requests
from hibp_py.utils import load_config

config = load_config()

API = {
    'endpoint': 'https://haveibeenpwned.com/api/v3/',
    'headers': {
        'User-Agent': 'hibp-py',
        'hibp-api-key': config['API_KEY']
    }
}


class AuthenticationError(Exception):
    """Raised when authentication fails"""
    pass


class ServerError(Exception):
    """Raised when the server returns an error"""
    pass


class RateLimitError(Exception):
    """Raised when the rate limit is exceeded"""
    pass


def get_breaches(account):
    """
    Get all breaches for an account
    :param account: Account to check
    :return: List of breaches
    """
    response = requests.get(
        f'{API["endpoint"]}breachedaccount/{account}', headers=API['headers'])

    if response.status_code == 200:
        return response.json()
    elif response.status_code == 401:
        raise AuthenticationError('Invalid API key')
    elif response.status_code == 404:
        return {}
    elif response.status_code == 429:
        raise RateLimitError(
            'Rate limit exceeded. Please wait and try again later.')
    else:
        raise ServerError(
            f'Server returned error code: {response.status_code}')


def get_breach(breach_name):
    response = requests.get(
        f'{API["endpoint"]}breach/{breach_name}', headers=API['headers'])

    if response.status_code == 200:
        return response.json()
    else:
        raise ServerError(
            f'Server returned error code: {response.status_code}')



from argparse import ArgumentParser, Namespace


def parse_args() -> Namespace:
    args = ArgumentParser()

    args.add_argument('-iL', '--input-list', type=str,
                      help='Input list of emails or haveibeenpwned JSON file')

    args.add_argument('-o', '--output', type=str,
                      default='output.json', help='Output file name')

    args.add_argument('-D', '--active-directory', type=str,
                      help='Active Directory domain')

    args.add_argument('-e', '--email-config', type=str,
                      help='Email configuration file')

    return args.parse_args()



import json
import os


DATA_DIR = './data'
USERS_FILE = f'{DATA_DIR}/users.json'
BREACHES_FILE = f'{DATA_DIR}/breaches.json'


def get_account_breaches(account):
    """Gets all breaches for an account from the database

    Args:
        account (str): The account to check for breaches

    Returns:
        list: A list of breach names
    """
    if os.path.isfile(USERS_FILE):
        with open(USERS_FILE, 'r', encoding='utf-8') as f:
            data = json.load(f)

        if account in data:
            return data[account]

    return []


def get_breach(breach_name):
    """Gets a breach from the database

    Args:
        breach_name (str): The breach to get

    Returns:
        dict: The breach data
    """

    if os.path.isfile(BREACHES_FILE):
        with open(BREACHES_FILE, 'r', encoding='utf-8') as f:
            data = json.load(f)

        if breach_name in data:
            return data[breach_name]

    return {}


def write_breach(user, breach):
    """Writes a breach to the database

    Args:
        user (str): The user to write the breach for
        breach (dict): The breach data

    Returns:
        bool: True if the breach is new, False otherwise
    """
    new_breach = False

    if not os.path.isdir(DATA_DIR):
        os.makedirs(DATA_DIR)

    if os.path.isfile(USERS_FILE):
        with open(USERS_FILE, 'r', encoding='utf-8') as f:
            users = json.load(f)
    else:
        users = {}

    if user not in users:
        new_breach = True
        users[user] = [breach['Name']]
    elif breach['Name'] not in users[user]:
        new_breach = True
        users[user].append(breach['Name'])

    with open(USERS_FILE, 'w', encoding='utf-8') as f:
        json.dump(users, f, sort_keys=True)

    if os.path.isfile(BREACHES_FILE):
        with open(BREACHES_FILE, 'r', encoding='utf-8') as f:
            data = json.load(f)
    else:
        data = {}

    if breach['Name'] not in data:
        data[breach['Name']] = breach

        with open(BREACHES_FILE, 'w', encoding='utf-8') as f:
            json.dump(data, f, sort_keys=True)

    return new_breach



import json
import smtplib
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.utils import formataddr
from hibp_py.utils import load_config


config = load_config()


SERVER = config['SMTP_SERVER']
PORT = config['SMTP_PORT']
FROM_NAME = config['FROM_NAME']
FROM_EMAIL = config['FROM_EMAIL']
SUBJECT = config['SUBJECT']


def send_email(email, body):
    """Send an email

    Args:
        email (str): The recipient's email address
        body (str): The email body
    """
    if 'TEST_RECIPIENT' in config.keys():
        email = config['TEST_RECIPIENT']

    message = MIMEMultipart('alternative')
    message['Subject'] = SUBJECT
    message['From'] = formataddr((FROM_NAME, FROM_EMAIL))
    message['To'] = email

    message.attach(MIMEText(body, 'html'))

    server = smtplib.SMTP()
    server.connect(SERVER, PORT)
    server.sendmail(FROM_EMAIL, email, message.as_string())
    server.quit()


def create_body(email, breachNames):
    """Creates the email body

    Args:
        email (str): The recipient's email address
        breachNames (list): A list of breach names

    Returns:
        str: The email body
    """
    with open('input/email.html', 'r') as file:
        body = file.read()

    breaches_list = ''
    breached_data = ''
    data_classes = []

    breaches = {}

    with open(f'data/breaches.json', 'r') as file:
        data = json.load(file)

        for breachName in breachNames:
            breaches[breachName] = data[breachName]

    for breachName in breaches:
        if breaches[breachName]:
            breach = breaches[breachName]

            if breach['Domain']:
                breaches_list += f'<li><a href="https://{breach["Domain"]}"' + \
                    f'>{breach["Title"]}</a></li>'
            else:
                breaches_list += f'<li>{breach["Title"]}</li>'

            if breach["DataClasses"]:
                for data_class in breach["DataClasses"]:
                    if data_class not in data_classes:
                        data_classes.append(data_class)
                        breached_data += f'<li>{data_class}</li>'

    body = body.replace('{email}', email)
    body = body.replace('{breaches}', breaches_list)
    body = body.replace('{breached_data}', breached_data)

    return body



import datetime
import json
import os
import time


def load_config(path="config.json"):
    """Load configuration from a JSON file

    Args:
        path (str, optional): Path to the JSON file. Defaults to "config.json".

    Returns:
        dict: Configuration
    """
    with open(path, 'r', encoding='utf-8') as f:
        return json.load(f)


def handle_rate_limit(logger):
    """Handle rate limit exceeded"""
    logger.log_event('Rate limit exceeded. Waiting 6 seconds...', 'WARNING')
    time.sleep(6)


class Logger:
    def __init__(self, path="events.log", rotation_size=1000):
        self.path = path
        self.rotation_size = rotation_size

    def log_event(self, event, severity="INFO"):
        """Log an event to a file

        Args:
            event (str): Event to log
            severity (str, optional): Severity of the event. Defaults to "INFO".
        """
        event_str = f'{datetime.datetime.now()} [{severity}] {event}'

        print(event_str)

        with open(self.path, 'r+', encoding='utf-8') as f:
            if len(f.readlines()) >= self.rotation_size:
                self.rotate()

            f.write(event_str + '\n')

    def rotate(self):
        """Rotate the log file"""
        file_path_no_ext = self.path.split('.')[0]

        # Rename the current log file to events.log.<number>
        for i in range(9, -1, -1):
            path = f'{file_path_no_ext}{i > 0 and f".{i}" or ""}.log'

            if os.path.exists(path):
                os.rename(path, f'{file_path_no_ext}.{i + 1}.log')

        with open(self.path, 'w', encoding='utf-8') as f:
            f.write('')


if __name__ == "__main__":
    config = load_config()
    print(config)



import win32api
import win32security


class ActiveDirectoryConnection():
    def __init__(self) -> None:
        import win32api
        import win32security

        # Get the process handle for the current process
        process_handle = win32api.GetCurrentProcess()

        # Get the token handle for the process
        token_handle = win32security.OpenProcessToken(
            process_handle, win32security.TOKEN_READ)

        # Get the SID for the current user
        user_sid = win32security.GetTokenInformation(
            token_handle, win32security.TokenUser)[0]

        # Get the user object from Active Directory using the user's SID
        user_obj = win32security.LookupAccountSid(None, user_sid)

        # Print the user's domain, username, and full name
        print(
            f"Domain: {user_obj[0]}\nUsername: {user_obj[1]}\nFull Name: {user_obj[2]}")



import hibp_py.api as api
import hibp_py.args as args
import hibp_py.db as db
import hibp_py.mail as mail
import hibp_py.utils as utils
import time


def main():
    config = utils.load_config()
    logger = utils.Logger()

    if 'LOG_PATH' in config:
        logger.__setattr__('path', config['LOG_PATH'])

    if 'ROTATION_SIZE' in config:
        logger.__setattr__('rotation_size', config['ROTATION_SIZE'])

    print('Welcome to HIBP Python!')

    arguments = args.parse_args()

    if arguments.input_list:
        with open(arguments.input_list, 'r', encoding='utf-16') as f:
            accounts = f.read().splitlines()

        for account in accounts:
            logger.log_event(f'Checking {account}...', 'INFO')
            success = False
            new_breaches = []

            while not success:
                try:
                    breaches = api.get_breaches(account)
                    success = True
                except api.RateLimitError as e:
                    utils.handle_rate_limit(logger)
                    time.sleep(6)

            for breach in breaches:

                breach_name = breach['Name']
                success = False

                try:
                    breach_data = db.get_breach(breach_name)

                    if not breach_data:
                        while not success:
                            try:
                                breach_data = api.get_breach(breach_name)
                                success = True
                            except api.RateLimitError as e:
                                utils.handle_rate_limit(logger)
                                time.sleep(6)

                    if db.write_breach(account, breach_data):
                        logger.log_event(
                            f'New breach found: {breach_name}', 'BREACH')

                        new_breaches.append(breach_name)

                    success = True
                except api.RateLimitError as e:
                    utils.handle_rate_limit(logger)

            if new_breaches:
                mail.send_email(account, mail.create_body(
                    account, new_breaches))

            time.sleep(6)


if __name__ == '__main__':
    main()
